﻿using System;
using System.Net.Http;
using System.Text;
using System.Text.Json;
using System.Threading.Tasks;

namespace TestProject;

/// <summary>
/// 프로그램
/// </summary>
public class Program
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Private

    #region OLLAMA 요청 보내기 (비동기) - SendOllamaRequestAsync(httpClient, ollamaRequest)

    /// <summary>
    /// OLLAMA 요청 보내기 (비동기)
    /// </summary>
    /// <param name="httpClient">HTTP 클라이언트</param>
    /// <param name="ollamaRequest">OLLAMA 요청</param>
    /// <returns>OLLAMA 응답 태스크</returns>
    private static async Task<OllamaResponse> SendOllamaRequestAsync(HttpClient httpClient, OllamaRequest ollamaRequest)
    {
        string json = JsonSerializer.Serialize
        (
            ollamaRequest,
            new JsonSerializerOptions { PropertyNamingPolicy = JsonNamingPolicy.CamelCase }
        );

        StringContent stringContent = new StringContent(json, Encoding.UTF8, "application/json");

        HttpResponseMessage response = await httpClient.PostAsync("/api/generate", stringContent);

        response.EnsureSuccessStatusCode();

        string responseBody = await response.Content.ReadAsStringAsync();

        return JsonSerializer.Deserialize<OllamaResponse>
        (
            responseBody,
            new JsonSerializerOptions { PropertyNamingPolicy = JsonNamingPolicy.CamelCase }
        );
    }

    #endregion
    #region 프로그램 시작하기 - Main()

    /// <summary>
    /// 프로그램 시작하기
    /// </summary>
    /// <returns>태스크</returns>
    private static async Task Main()
    {
        string baseURL = "http://localhost:11434";
        
        using var httpClient = new HttpClient();

        httpClient.BaseAddress = new Uri(baseURL);

        OllamaRequest ollamaRequest = new()
        {
            Model  = "bnksys/eeve-yanolja-v1:latest",
            Prompt = "여기어때와 야놀자의 차이점을 알려주세요.",
            Stream = false
        };

        try
        {
            OllamaResponse ollamaResponse = await SendOllamaRequestAsync(httpClient, ollamaRequest);

            Console.WriteLine($"질문 : {ollamaRequest.Prompt   }");
            Console.WriteLine($"응답 : {ollamaResponse.Response}");
        }
        catch(Exception exception)
        {
            Console.WriteLine($"오류 발생 : {exception.Message}");
        }
    }

    #endregion
}